/*------------------------------------------------------------------------------
    GALE. Fine-grained parallel Genetic Algorithms for Data Mining.
    Copyright (C) 2002  Xavier Llor <xllora@illigal.ge.uiuc.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


package edu.uiuc.ge.illigal.gale.data;

/**
 * This class is a view over a certain InstanceMemory.
 *
 * @author  Xavier Llor&agrave; &lt;<a href="mailto:xllora@illigal.ge.uiuc.edu">xllora@illigal.ge.uiuc.edu</a>&gt;
 * @version 0.9alpha
 * @since 0.9alpha
 */

public class CVView 
implements OrderedSet
{
   //---------------------------------------------------------------------

	/** The folds for cross-validation */
	protected OrderedSet [] osaData;

	/** The number of instances in the view (calculated field) */
	protected int iSize;

	/** The CVView sets */
	protected OrderedSet [] osac;

	/** The thresholds for the sample recovery */
	protected int [] iac;

   //---------------------------------------------------------------------

	/** 
	 * Builds a view over the OrderSet given.
	 *
    * @param osa The folds for cross-validation
	 * @param iFold The folder to leave out
    */
   public CVView ( OrderedSet [] osa, int iFold )
   {
		osaData = osa;

		osac = new OrderedSet[osa.length-1];
		iac  = new int[osa.length-1];

		for ( int i=0, j=0, iMax=osa.length ; i<iMax ; i++ )
			if ( i!=iFold ) {
				osac[j] = osa[i];
				iac[j]  = osa[i].getSize();	
				j++;
			}

		for ( int i=1, iMax=osac.length ; i<iMax ; i++ )
			iac[i] += iac[i-1];
				
		iSize = iac[iac.length-1];
   }

   //---------------------------------------------------------------------

	// OrderedSet implementation
   public Instance getInstance ( int iPos )
	{
		int i=0, j=0;

		for ( int iMax=osac.length ; i<iMax && iac[i]<=iPos ; i++ );
		
		j = (i>0)?iPos-iac[i-1]:iPos;

		if ( iPos>=0 && iPos<iSize )
			return osac[i].getInstance(j);
		else
			return null;
	}

   //---------------------------------------------------------------------

   // OrderedSet implementation
	public int getSize ()
   {
		return iSize;
	}

   //---------------------------------------------------------------------
}
